unit QuickToolsMain;

interface

{$I QuickTools.inc}

uses
  Windows,
  Messages,
  ExtCtrls,
  ComCtrls,
  SysUtils,
  Forms,
  Classes,
  Menus;

type
  TQuickToolsMainObject = class(TObject)
//  private
  public
    FTimer:TTimer;
    FInTimer:Boolean;
    FOldResizeHandler:TNotifyEvent;

//    FullScreen:Boolean;
    DelphiStandardProjectClosed:Boolean;

    procedure OnTimer(Sender: TObject);
    function FindMenuItem(Menu:TMenuItem;Name:String):TMenuItem;

//  protected
//  public
    constructor Create;
    destructor Destroy; override;

    procedure AlignMainForm;
    procedure AlignPropertyInspector;
    procedure AlignEditorForm;
    procedure AlignProjectManagerForm;
//    procedure AlignPackageWindows;
//    procedure AlignMessageWindow;
    procedure CloseStandardProject;
    procedure OnResizeComponentPalette(Sender:TObject);
  published
  end; { TQuickToolsMainObject }

var
  QuickToolsMainObject:TQuickToolsMainObject;

procedure RestoreFormsBruteForce;

implementation

uses
  {$IFDEF Delphi6}
  DesignIntf, DesignWindows, ExptIntf,
  {$ELSE}
  ExptIntf,
  DsgnIntf,
  {$ENDIF}
  QuickToolsGlobal,
  QuickToolsDelphiMainForm;

function TQuickToolsMainObject.FindMenuItem(Menu:TMenuItem;Name:String):TMenuItem;
var
  j:Integer;
begin
  Result:=nil;
  for j:=0 to Menu.Count-1 do
  begin
    if Menu.Items[j].Name=Name then
      Result:=Menu.Items[j]
    else if Menu.Items[j].Count>0 then
      Result:=FindMenuItem(Menu.Items[j],Name);
    if Result<>nil then
      break;
  end;
end;

constructor TQuickToolsMainObject.Create;
begin
  inherited Create;

//  FullScreen:=true;
  DelphiStandardProjectClosed:=false;

  if Options.HideCaptions then
  begin
    AlignMainForm;
    qtDelphiMainForm.Show;
  end;

  FTimer:=TTimer.Create(nil);
  FTimer.OnTimer:=OnTimer;
  FTimer.Interval:=1000;
end;

destructor TQuickToolsMainObject.Destroy;
{$IFDEF Delphi6}
var
  i:Integer;
{$ENDIF}
begin
  {$IFDEF Delphi6}
  for i:=0 to Application.MainForm.ComponentCount-1 do
  begin
    if Application.MainForm.Components[i].Name='TabControl' then
    with TTabControl(Application.MainForm.Components[i]) do
    begin
      OnResize:=FOldResizeHandler;
      break;
    end;
  end;
  {$ENDIF}
  FTimer.Free;
  inherited Destroy;
end;

procedure TQuickToolsMainObject.OnTimer(Sender: TObject);
begin
//  if FInTimer then
//    exit;
//  FInTimer:=true;
  AlignPropertyInspector;
  AlignProjectManagerForm;
  AlignEditorForm;
//  AlignMessageWindow;
//  AlignPackageWindows;
  CloseStandardProject;
end;

procedure TQuickToolsMainObject.OnResizeComponentPalette(Sender: TObject);
var
  AHeight: Integer;
  SenderControl: TTabControl;
begin
  SenderControl := Sender as TTabControl;

  AHeight := SenderControl.Height -
             (SenderControl.DisplayRect.Bottom - SenderControl.DisplayRect.Top) + 29;

  SenderControl.Constraints.MinHeight := AHeight;
  SenderControl.Parent.Constraints.MaxHeight := AHeight;
end;

procedure TQuickToolsMainObject.AlignMainForm;
{$IFDEF Delphi6}
var
  i:Integer;
{$ENDIF}
begin
  with Application.MainForm do
  begin
    BorderStyle:=bsNone;
    SetBounds(0,0,
      GetSystemMetrics(SM_CXMaximized)-2*GetSystemMetrics(SM_CXFRAME),0);
//    Left:=0;
//    Top:=0;
//    Width:=GetSystemMetrics(SM_CXMaximized)-2*GetSystemMetrics(SM_CXFRAME);
//    Height:=0;
    //SM_CXFULLSCREEN
    //Screen.Width;
    //TabControl:=TTabControl(FindComponent('TabControl'));
    //if (TabControl<>nil) then
    //  TabControl.HotTrack:=QTC.HotTrack;
    {$IFDEF Delphi6}
    for i:=0 to Application.MainForm.ComponentCount-1 do
    begin
      if Application.MainForm.Components[i].Name='TabControl' then
      with TTabControl(Application.MainForm.Components[i]) do
      begin
        MultiLine:=true;
        Style:=tsFlatButtons;
        HotTrack:=true;
        RaggedRight:=true;
        FOldResizeHandler:=OnResize;
        OnResize:=OnResizeComponentPalette;
        break;
      end;
    end;
    {$ENDIF}
  end;
  //Application.HookMainWindow(MainWindowHook);
end;

procedure TQuickToolsMainObject.CloseStandardProject;
var
  MenuItem:TMenuItem;
  i:Integer;
begin
//TODO: StandardProjekt schlieen.
  if DelphiStandardProjectClosed then
    exit;
  DelphiStandardProjectClosed:=true;
  if not Options.CloseDefault then
    exit;
  if (ExtractFileName(ToolServices.GetProjectName)<>'Project1.dpr') and
     (ExtractFileName(ToolServices.GetProjectName)<>'Project1.cpp') then
    exit;
//  if ExtractFileName((BorlandIDEServices as IOTAProject).FileName)<>'Project1.dpr' then
//    exit;

  MenuItem:=nil;
  for i:=0 to Application.MainForm.ComponentCount-1 do
  begin
    if Application.MainForm.Components[i] is TMainMenu then
    begin
      MenuItem:=FindMenuItem(TMainMenu(Application.MainForm.Components[i]).Items,'FileCloseAllItem');
    end;
  end;
  if MenuItem<>nil then
  begin
    MenuItem.Click;
  end;
end;

procedure TQuickToolsMainObject.AlignPropertyInspector;
var
  i:Integer;
  {$IFDEF Delphi6}
  j:Integer;
  {$ENDIF}
begin
  if not Options.HideCaptions then
    exit;
  // Search for object-inspector, make borderstyle to bsnone and align it
  // left at the border of screen.
  {$IFDEF Delphi6}
  j:=Application.MainForm.Height;
  for i:=0 to Screen.FormCount-1 do
  begin
    if Screen.Forms[i].Name='ObjectTreeView' then
    with Screen.Forms[i] do
    begin
      Top:=Application.MainForm.Height;
      Left:=Application.MainForm.Left;
      Height:=GetSystemMetrics(SM_CYFULLSCREEN)-Top-GetSystemMetrics(SM_CYMINIMIZED)-Options.ObjectInspectorHeight;
      j:=Top+Height;

      BorderStyle:=bsNone;
      UseDockManager:=false;
      DockSite:=false;
      break;
    end;
  end;
  {$ENDIF}
  for i:=0 to Screen.FormCount-1 do
  begin
    if Screen.Forms[i].Name='PropertyInspector' then
    with Screen.Forms[i] do
    begin
      {$IFDEF Delphi6}
      Top:=j;
      Height:=GetSystemMetrics(SM_CYFULLSCREEN)-Top-GetSystemMetrics(SM_CYMINIMIZED);
      {$ELSE}
      Top:=Application.MainForm.Height;
      Height:=Options.ObjectInspectorHeight;
      {$ENDIF}
      Left:=Application.MainForm.Left;
      //Height:=Top

      BorderStyle:=bsNone;
      UseDockManager:=false;
      DockSite:=false;
      break;
    end;
  end;
end;

procedure TQuickToolsMainObject.AlignEditorForm;
var
  i: Integer;
  j:Integer;
  FLeft:Integer;
begin
  if not Options.MaximizeEditor then
    exit;
  for i:=0 to Screen.FormCount-1 do
  begin
    if Screen.Forms[i].Name='EditWindow_0' then
    with Screen.Forms[i] do
    begin
      FLeft:=Application.MainForm.Left;
      if WindowState<>wsMaximized then
        for j := 0 to Screen.FormCount-1 do
          if (Screen.Forms[j].Name='PropertyInspector') and
             (Screen.Forms[j].Visible) then
          begin
            FLeft:=Screen.Forms[j].Left+Screen.Forms[j].Width;
            break;
          end;
      SetBounds(
        FLeft,
        Application.MainForm.Height,
        Application.MainForm.Width-FLeft,
        Screen.Height-Top-GetSystemMetrics(SM_CYMINIMIZED));
      {$IFDEF Delphi6}
        for j:=0 to Screen.Forms[i].ComponentCount-1 do
          if Screen.Forms[i].Components[j].Name='TabControl' then
          with TTabControl(Screen.Forms[i].Components[j]) do
          begin
            Style:=tsFlatButtons;
            break;
          end;
      {$ENDIF}
    end;
  end;
end;

procedure TQuickToolsMainObject.AlignProjectManagerForm;
var
  i: Integer;
begin
  if not Options.HideCaptions then
    exit;
  for i:=0 to Screen.FormCount-1 do
  begin
    if Screen.Forms[i].Name='ProjectManagerForm' then
    with Screen.Forms[i] do
    begin
      Visible:=true;
      BorderStyle:=bsNone;
      Left:=Application.MainForm.Left;
      Top:=Screen.Height-Height-GetSystemMetrics(SM_CYMINIMIZED);
    end;
  end; 
end;

//procedure TQuickToolsMainObject.AlignPackageWindows;
//var
//  i:Integer;
//  j:Integer;
//  vTop:Integer;
//  Form:TForm;
//  found : boolean;
//begin
//  vTop:=Application.MainForm.Height;
//  j := 0;
//  found:=true;
//  Form:=nil;
//  while found do
//  begin
//    found:=false;
//    for i:=0 to Screen.FormCount-1 do
//    begin
//      if ((Screen.Forms[i].Name='PackageEditorForm') and (j=0)) or
//         (Screen.Forms[i].Name='PackageEditorForm_'+Trim(IntToStr(j))) then
//      begin
//        Form:=Screen.Forms[i];
//        found:=true;
//        break;
//      end;
//    end;
//    if found then
//    with Form do
//    begin
//      Visible:=true;
//      Top:=vTop;
//      vTop:=vTop+Height;
//      Left:=Application.MainForm.Width-Width;
//    end;
//    j := j + 1;
//  end;
//  //if (Form.Top+Form.Height)<(Screen.Height-GetSystemMetrics(SM_CYMINIMIZED)) then
//    Form.Height:=Screen.Height-GetSystemMetrics(SM_CYMINIMIZED)-Form.Top;
//end;

//procedure TQuickToolsMainObject.AlignMessageWindow;
//var
//  i:Integer;
//begin
//  for i:=0 to Screen.FormCount-1 do
//  begin
//    if Screen.Forms[i].Name='MsgWindow' then
//    with Screen.Forms[i] do
//    begin
//      begin
//        //if docksite then
//        //messagebox(0,'true',pchar(Screen.Forms[i].Name),mb_ok)
//        //else
//        //messagebox(0,'false',pchar(Screen.Forms[i].Name),mb_ok);
////        Top:=Application.MainForm.Height;
////        Left:=Application.MainForm.Left;
////        Height:=GetSystemMetrics(SM_CYFULLSCREEN)-Top-GetSystemMetrics(SM_CYMINIMIZED);
//      end;
////      BorderStyle:=bsNone;
////      UseDockManager:=false;
////      DockSite:=false;
//      break;
//    end;
//  end;
//end;

procedure RestoreForms;
var
  i:Integer;
  found:Boolean;
begin
  found:=false;
  for i:=0 to Screen.FormCount-1 do
    if Screen.Forms[i].Name='ProjectOptionsDialog' then
    begin
      found:=true;
      break;
    end;
  // restoring of forms
  if found then
  begin
    Application.MainForm.BorderStyle:=bsSizeable;
    for i:=0 to Screen.FormCount-1 do
      if (Screen.Forms[i].Name='ProjectManagerForm') or
         (Screen.Forms[i].Name='PropertyInspector') then
        Screen.Forms[i].BorderStyle:=bsSizeToolWin;
  end;
end;

procedure RestoreFormsBruteForce;
var
  i:Integer;
begin
  Application.MainForm.BorderStyle:=bsSizeable;
  for i:=0 to Screen.FormCount-1 do
  begin
    if (Screen.Forms[i].Name='ProjectManagerForm') or
       (Screen.Forms[i].Name='PropertyInspector') then
      Screen.Forms[i].BorderStyle:=bsSizeToolWin;
    if Screen.Forms[i].Name='EditWindow_0' then
    with Screen.Forms[i] do
    begin
      BorderStyle:=bsSizeable;
      BorderIcons:=BorderIcons+[biMaximize];
    end;
  end;
end;

initialization
  if not ProjOptDialog then
    QuickToolsMainObject:=TQuickToolsMainObject.Create
  else
    Application.MessageBox('Please restart Delphi to activate QuickTools!','QuickTools',MB_OK);
finalization
  QuickToolsMainObject.Free;
  RestoreForms;
end.

