unit QuickToolsMenu;

interface

uses
  Windows, Classes, ToolsApi, SysUtils, Menus, Forms, Dialogs, Controls;

type
  TQuickToolsMenuExpert=class(TNotifierObject,IOTAWizard)
  private
    FMainMenu:TMainMenu;
    FToolsMenu:TMenuItem;
    FOptionsItem:TMenuItem;
  protected
    // IOTAWizard methods
    function GetIDString:string;
    function GetName:string;
    function GetState:TWizardState;
    procedure Execute;
  public
    constructor Create;
    destructor Destroy; override;
    procedure OptionsClick(Sender:TObject);
  end;

procedure Register;

implementation

uses
  QuickToolsGlobal,
  QuickToolsOptions,
  QuickToolsIconChanger,
  QuickToolsMain,
  QuickToolsDelphiMainForm;

var
  Index:Integer;

procedure Register;
begin
  Index:=(BorlandIDEServices as IOTAWizardServices).AddWizard(TQuickToolsMenuExpert.Create);
end;

{ TQuickToolsMenuExpert }

constructor TQuickToolsMenuExpert.Create;
begin
  inherited Create;
  FMainMenu   :=(BorlandIDEServices as INTAServices).MainMenu;
  FToolsMenu  :=FMainMenu.Items.Find('Tools');
  FOptionsItem:=FMainMenu.FindComponent(GetName) as TMenuItem;
  if FOptionsItem=nil then
  begin
    FOptionsItem:=TMenuItem.Create(FToolsMenu);

    FOptionsItem.Caption:='&QuickTools Options...';
    FOptionsItem.OnClick:=OptionsClick;
    FOptionsItem.Name   :=GetName;

    FToolsMenu.Insert(0,FOptionsItem);
  end;
end;

destructor TQuickToolsMenuExpert.Destroy;
var
  i:Integer;
begin
  i:=0;
  while i<FToolsMenu.Count do
  begin
    if AnsiSameCaption(FToolsMenu.Items[i].Caption,GetName) then
    begin
      FToolsMenu.Remove(FToolsMenu.Items[i]);
    end
    else
      Inc(i);
  end;
  if FOptionsItem<>nil then
  begin
    FOptionsItem.Free;
  end;
  inherited Destroy;
end;

procedure TQuickToolsMenuExpert.Execute;
begin
  // do nothing...
end;

function TQuickToolsMenuExpert.GetIDString: string;
begin
  result:='dev0.QuickToolsMenuExpert';
end;

function TQuickToolsMenuExpert.GetName: string;
begin
  result:='QuickToolsOptions';
end;

function TQuickToolsMenuExpert.GetState: TWizardState;
begin
  result:=[wsEnabled];
end;

procedure TQuickToolsMenuExpert.OptionsClick(Sender: TObject);
begin
  fmOptions:=TfmOptions.Create(Application.MainForm);
  if fmOptions.ShowModal=mrOK then
  begin
    // apply options now!
    Options.CloseDefault:=fmOptions.ckCloseDefault.Checked;
    Options.HideCaptions:=fmOptions.ckHideCaptions.Checked;
    Options.MaximizeEditor:=fmOptions.ckMaximizeEditor.Checked;
    Options.CreateCompDate:=fmOptions.ckCreateCompDate.Checked;
    Options.ChangeIcon:=fmOptions.ckChangeIcon.Checked;
    Options.ObjectInspectorHeight:=StrToInt(fmOptions.edOIHeight.Text);
    WriteOptionsToRegistry;
    RestoreFormsBruteForce;

    // update some things now...
    ChangeIcon;
    if not Options.HideCaptions then
    begin
      qtDelphiMainForm.Hide;
    end
    else
    begin
      QuickToolsMainObject.AlignMainForm;
      qtDelphiMainForm.Show;
    end;
  end;
  fmOptions.Free;
end;

initialization
finalization
  (BorlandIDEServices as IOTAWizardServices).RemoveWizard(Index);
end.

